package org.opengroup.osdu.notification.di;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.util.IServiceAccountJwtClient;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class CredentialHeadersProviderTest {

    @Mock
    private HttpServletRequest httpRequest;

    @Mock
    private IPubsubRequestBodyExtractor pubsubRequestBodyExtractor;

    @Mock
    private IServiceAccountJwtClient serviceAccountJwtClient;

    @InjectMocks
    private CredentialHeadersProvider headersProvider;

    @Before
    public void setup() {
        MockitoAnnotations.initMocks(CredentialHeadersProviderTest.this);
    }

    @Test
    public void testGetIsOk() throws Exception {
        when(httpRequest.getMethod()).thenReturn(RequestMethod.GET.toString());
        assertNotNull(headersProvider.getObject());
    }

    @Test
    public void getObject_patchTest() throws Exception {
        when(httpRequest.getMethod()).thenReturn(RequestMethod.PATCH.toString());
        Map<String, String> mockAttributes = new HashMap<>();
        mockAttributes.put("data-partition-id", "opendes");
        when(pubsubRequestBodyExtractor.extractAttributesFromRequestBody()).thenReturn(mockAttributes);
        when(serviceAccountJwtClient.getIdToken("opendes")).thenReturn("sampleAuthToken");

        DpsHeaders dpsHeaders = headersProvider.getObject();
        assertNotNull(dpsHeaders);
        assertEquals(dpsHeaders.getHeaders().get("content-type"), "application/json");
        assertEquals(dpsHeaders.getHeaders().get("authorization"), "sampleAuthToken");
        assertEquals(dpsHeaders.getHeaders().get("data-partition-id"), "opendes");
    }

}