// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.api;

import com.sun.jersey.api.client.ClientResponse;
import org.junit.*;
import org.opengroup.osdu.notification.util.AzureTestUtils;
import org.opengroup.osdu.notification.util.Config;

import static org.junit.Assert.assertEquals;

@Ignore
public class TestPubsubEndpointHMAC extends PubsubEndpointHMACTests {

    @BeforeClass
    public static void classSetup() throws Exception {
        PubsubEndpointHMACTests.classSetup();
    }

    @AfterClass
    public static void classTearDown() throws Exception {
    }

    @Before
    @Override
    public void setup() throws Exception {
        this.testUtils = new AzureTestUtils();
    }

    @After
    @Override
    public void tearDown() throws Exception {
        this.testUtils = null;
    }

    @Test
    @Override
    public void should_return400_when_makingHttpRequestWithoutToken() throws Exception {
        ClientResponse response =  descriptor.run(getArg(), "");
        assertEquals(error(response.getEntity(String.class)), 403, response.getStatus());
    }

    @Test
    @Override
    public void should_return307_when_makingHttpRequest() throws Exception {
        // The requirement of http support is under discussion.
        // If HTTP is a need, corresponding infra changes will be required for this test to function.
        return;
    }

    // For the following 403 is the expected result.
    // Tracking in Issue: https://community.opengroup.org/osdu/platform/system/notification/-/issues/17
    @Test
    @Override
    public void should_return401_when_accessingWithNoAccessCredentials() throws Exception{
        ClientResponse response = descriptor.run(getArg(), testUtils.getNoAccessToken());
        assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
    }

    @Test
    @Override
    public void should_return401_when_noAccessOnCustomerTenant() throws Exception {
        ClientResponse response = descriptor.runOnCustomerTenant(getArg(), getOsduTenantAdminCredentials());
        assertEquals(error(	response.getEntity(String.class)), 403, response.getStatus());
    }

    @Test
    @Override
    public void should_return401_when_accessingWithEditorCredentials() throws Exception{
        ClientResponse response = descriptor.run(getArg(), testUtils.getNoAccessToken());
        assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
    }

    @Test
    @Override
    public void should_return401_when_accessingWithAdminCredentials() throws Exception{
        ClientResponse response = descriptor.run(getArg(), testUtils.getNoAccessToken());
        assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
    }

    @Test
    @Override
    public void should_return20X_when_usingCredentialsWithOpsPermission() throws Exception {
        this.createResource();

        try {
            ClientResponse response = descriptor.run(this.getArg(), this.testUtils.getOpsToken());
            Assert.assertEquals(this.error(response.getStatus() == 204 ? "" : (String)response.getEntity(String.class)), (long)this.expectedOkResponseCode(), (long)response.getStatus());
            Assert.assertEquals("GET, POST, PUT, DELETE, OPTIONS, HEAD, PATCH", response.getHeaders().getFirst("Access-Control-Allow-Methods"));
            Assert.assertEquals("access-control-allow-origin, origin, content-type, accept, authorization, data-partition-id, correlation-id, appkey", response.getHeaders().getFirst("Access-Control-Allow-Headers"));
            Assert.assertEquals("*", response.getHeaders().getFirst("Access-Control-Allow-Origin"));
            Assert.assertEquals("true", response.getHeaders().getFirst("Access-Control-Allow-Credentials"));
            Assert.assertEquals("DENY", response.getHeaders().getFirst("X-Frame-Options"));
            Assert.assertEquals("1; mode=block", response.getHeaders().getFirst("X-XSS-Protection"));
            Assert.assertEquals("nosniff", response.getHeaders().getFirst("X-Content-Type-Options"));
            Assert.assertEquals("no-cache, no-store, must-revalidate", response.getHeaders().getFirst("Cache-Control"));
            Assert.assertEquals("default-src 'self'", response.getHeaders().getFirst("Content-Security-Policy"));
            Assert.assertEquals("max-age=31536000; includeSubDomains", response.getHeaders().getFirst("Strict-Transport-Security"));
            Assert.assertEquals("0", response.getHeaders().getFirst("Expires"));
        } finally {
            this.deleteResource();
        }

    }

    @Test
    @Override
    public void should_return20XResponseCode_when_makingValidHttpsRequest() throws Exception {
        this.createResource();

        try {
            ClientResponse response = descriptor.run(this.getArg(), this.testUtils.getOpsToken());
            Assert.assertEquals(this.error(""), (long)this.expectedOkResponseCode(), (long)response.getStatus());
            this.validate20XResponse(response, descriptor);
        } finally {
            this.deleteResource();
        }
    }
}